@@ -26,6 +26,7 @@
 #endif /* HAVE_CONFIG_H */
 
 #include <stdbool.h>
+#include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -40,8 +41,6 @@
 
 #include "manconfig.h"
 
-#include "pipeline.h"
-
 #include "debug.h"
 #include "encodings.h"
 #include "pathsearch.h"
@@ -591,7 +590,8 @@ out:
 
 /* Can we take this input encoding and produce this output encoding, perhaps
  * with the help of some iconv pipes? */
-static bool compatible_encodings (const char *input, const char *output)
+static bool ATTRIBUTE_PURE compatible_encodings (const char *input,
+						 const char *output)
 {
 	if (STREQ (input, output))
 		return true;
