@@ -78,6 +78,7 @@
 #include "cleanup.h"
 #include "debug.h"
 #include "fatal.h"
+#include "filenames.h"
 #include "glcontainers.h"
 #include "pipeline.h"
 #include "util.h"
@@ -102,10 +103,12 @@ error_t argp_err_exit_status = FAIL;
 static const char args_doc[] = N_("[SECTION...]");
 
 static struct argp_option options[] = {
-	{ "debug",		'd',	0,		0,	N_("emit debugging messages") },
-	{ "manpath",		'M',	N_("PATH"),	0,	N_("set search path for manual pages to PATH") },
-	{ "config-file",	'C',	N_("FILE"),	0,	N_("use this user configuration file") },
-	{ 0, 'h', 0, OPTION_HIDDEN, 0 }, /* compatibility for --help */
+	OPT ("debug", 'd', 0, N_("emit debugging messages")),
+	OPT ("manpath", 'M', N_("PATH"),
+	     N_("set search path for manual pages to PATH")),
+	OPT ("config-file", 'C', N_("FILE"),
+	     N_("use this user configuration file")),
+	OPT_HELP_COMPAT,
 	{ 0 }
 };
 
@@ -231,7 +234,8 @@ static int parse_for_sec (MYDBM_FILE dbf,
 	pipecmd *basecmd, *cmd;
 	datum key;
 	size_t arg_size, initial_bit;
-	int message = 1, first_arg;
+	bool message = true;
+	int first_arg;
 
 	basecmd = pipecmd_new (MAN);
 	pipecmd_clearenv (basecmd);
@@ -282,20 +286,20 @@ static int parse_for_sec (MYDBM_FILE dbf,
 #endif
 			if (*MYDBM_DPTR (content) != '\t') {
 #pragma GCC diagnostic pop
-				struct mandata entry;
+				struct mandata *entry;
 
-				split_content (dbf, MYDBM_DPTR (content),
-					       &entry);
+				entry = split_content (dbf,
+						       MYDBM_DPTR (content));
 
 				/* Accept if the entry is an ultimate manual
 				   page and the section matches the one we're
 				   currently dealing with */
-				if (entry.id == ULT_MAN &&
-				    strcmp (entry.sec, section) == 0) {
+				if (entry->id == ULT_MAN &&
+				    strcmp (entry->sec, section) == 0) {
 					if (message) {
 						printf (_("\nUpdating cat files for section %s of man hierarchy %s\n"),
 							section, manpath);
-						message = 0;
+						message = false;
 					}
 
 					arg_size += add_arg (cmd, key) + 1;
@@ -317,9 +321,7 @@ static int parse_for_sec (MYDBM_FILE dbf,
 				    	}
 				}
 
-				/* == MYDBM_DPTR (content), freed below */
-				entry.addr = NULL;
-				free_mandata_elements (&entry);
+				free_mandata_struct (entry);
 			}
 
 			/* we don't need the content ever again */
@@ -342,14 +344,14 @@ static int parse_for_sec (MYDBM_FILE dbf,
 	return 0;
 }
 
-static int check_access (const char *directory)
+static bool check_access (const char *directory)
 {
 	if (!CAN_ACCESS (directory, W_OK)) {
 		error (0, errno, _("cannot write within %s"), directory);
-		return 1;
+		return true;
 	}
 
-	return 0;
+	return false;
 }
 
 int main (int argc, char *argv[])
