@@ -20,6 +20,8 @@
  * Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include <stdbool.h>
+
 #include "gl_list.h"
 
 enum look_for_file_opts {
@@ -30,7 +32,7 @@ enum look_for_file_opts {
 
 /* globbing.c */
 extern gl_list_t look_for_file (const char *hier, const char *sec,
-				const char *unesc_name, int cat, int opts);
+				const char *unesc_name, bool cat, int opts);
 
 /* Expand path with wildcards into list of all existing directories. */
 extern gl_list_t expand_path (const char *path);
