@@ -22,4 +22,14 @@
 
 #include "gl_list.h"
 
-void manconv (struct pipeline *p, gl_list_t from, const char *to);
+#include "decompress.h"
+
+struct manconv_outbuf {
+	char *buf;
+	size_t len, max;
+};
+
+char *check_preprocessor_encoding (decompress *decomp, const char *to_code,
+				   char **modified_line);
+int manconv (decompress *decomp, gl_list_t from, const char *to,
+	     struct manconv_outbuf *outbuf);
