@@ -30,19 +30,22 @@
 
 #include "gl_array_list.h"
 #include "gl_xlist.h"
-#include "idpriv.h"
+#include "xalloc.h"
 #include "xvasprintf.h"
 
 #include "manconfig.h"
 
 #include "pipeline.h"
-#include "decompress.h"
+
+#include "appendstr.h"
 #include "glcontainers.h"
 #include "sandbox.h"
 #include "security.h"
 
+#include "decompress.h"
 #include "manconv.h"
 #include "manconv_client.h"
+#include "utf8.h"
 
 extern man_sandbox *sandbox;
 
@@ -54,13 +57,17 @@ struct manconv_codes {
 static void manconv_stdin (void *data)
 {
 	struct manconv_codes *codes = data;
-	pipeline *p;
+	decompress *decomp;
 
-	p = decompress_fdopen (dup (STDIN_FILENO));
-	pipeline_start (p);
-	manconv (p, codes->from, codes->to);
-	pipeline_wait (p);
-	pipeline_free (p);
+	decomp = decompress_fdopen (dup (STDIN_FILENO));
+	decompress_start (decomp);
+	if (manconv (decomp, codes->from, codes->to, NULL) != 0)
+		/* manconv already wrote an error message to stderr.  Just
+		 * exit non-zero.
+		 */
+		exit (FATAL);
+	decompress_wait (decomp);
+	decompress_free (decomp);
 }
 
 static void manconv_pre_exec (void *data)
@@ -81,28 +88,29 @@ static void free_manconv_codes (void *data)
 	free (codes);
 }
 
-void add_manconv (pipeline *p, const char *source, const char *target)
+void add_manconv (pipeline *p,
+		  const char *source_encoding, const char *target_encoding)
 {
 	struct manconv_codes *codes;
 	char *name;
 	pipecmd *cmd;
 
-	if (STREQ (source, "UTF-8") && STREQ (target, "UTF-8"))
+	if (STREQ (source_encoding, target_encoding))
 		return;
 
 	codes = xmalloc (sizeof *codes);
 	/* informational only; no shell quoting concerns */
 	name = xasprintf ("%s -f ", MANCONV);
 	codes->from = new_string_list (GL_ARRAY_LIST, true);
-	if (STREQ (source, "UTF-8")) {
-		gl_list_add_last (codes->from, xstrdup (source));
-		name = appendstr (name, source, (void *) 0);
+	if (STREQ (source_encoding, "UTF-8")) {
+		gl_list_add_last (codes->from, xstrdup (source_encoding));
+		name = appendstr (name, source_encoding, (void *) 0);
 	} else {
 		gl_list_add_last (codes->from, xstrdup ("UTF-8"));
-		gl_list_add_last (codes->from, xstrdup (source));
-		name = appendstr (name, "UTF-8:", source, (void *) 0);
+		gl_list_add_last (codes->from, xstrdup (source_encoding));
+		name = appendstr (name, "UTF-8:", source_encoding, (void *) 0);
 	}
-	codes->to = xasprintf ("%s//IGNORE", target);
+	codes->to = xasprintf ("%s//IGNORE", target_encoding);
 	/* informational only; no shell quoting concerns */
 	name = appendstr (name, " -t ", codes->to, (void *) 0);
 	if (quiet >= 2)
@@ -122,11 +130,11 @@ void add_manconv (pipeline *p, const char *source, const char *target)
 		char *sources = NULL;
 
 		cmd = pipecmd_new_args (MANCONV, "-f", (void *) 0);
-		GL_LIST_FOREACH_START (from, from_code) {
+		GL_LIST_FOREACH (from, from_code) {
 			sources = appendstr (sources, from_code, (void *) 0);
 			if (gl_list_next_node (from, from_node))
 				sources = appendstr (sources, ":", (void *) 0);
-		} GL_LIST_FOREACH_END (from);
+		}
 		pipecmd_arg (cmd, sources);
 		free (sources);
 		pipecmd_args (cmd, "-t", codes->to, (void *) 0);
@@ -143,3 +151,68 @@ void add_manconv (pipeline *p, const char *source, const char *target)
 	free (name);
 	pipeline_command (p, cmd);
 }
+
+/* Convert the result of in-process decompression to a target encoding.
+ *
+ * This converts the buffered result of decompression to a new buffer, then
+ * replaces the decompress object's buffer with the converted one for use by
+ * later stages of processing.
+ *
+ * Returns zero on success or non-zero on failure.
+ */
+int manconv_inprocess (decompress *d,
+		       const char *source_encoding,
+		       const char *target_encoding)
+{
+	gl_list_t from;
+	char *to;
+	struct manconv_outbuf outbuf;
+	int ret = 0;
+
+	if (STREQ (source_encoding, target_encoding))
+		return 0;
+
+	from = new_string_list (GL_ARRAY_LIST, true);
+	if (STREQ (source_encoding, "UTF-8"))
+		gl_list_add_last (from, xstrdup (source_encoding));
+	else {
+		if (STREQ (target_encoding, "UTF-8")) {
+			/* If the target encoding is UTF-8, then instead of
+			 * starting with trial conversion from UTF-8 to
+			 * UTF-8, we can start by simply performing UTF-8
+			 * validation, avoiding a copy.  (The source
+			 * encoding cannot be UTF-8 in this case, since we
+			 * already checked that the source and target
+			 * encodings are different.)
+			 */
+			if (utf8_validate_len (decompress_inprocess_buf (d),
+					       decompress_inprocess_len (d)))
+				goto out;
+		} else
+			gl_list_add_last (from, xstrdup ("UTF-8"));
+		gl_list_add_last (from, xstrdup (source_encoding));
+	}
+	to = xasprintf ("%s//IGNORE", target_encoding);
+
+	outbuf.len = 0;
+	/* UTF-8 uses at most four bytes per Unicode code point.  We assume
+	 * that this conversion will be no worse than 1:4.
+	 */
+	outbuf.max = decompress_inprocess_len (d) * 4;
+	outbuf.buf = xmalloc (outbuf.max);
+
+	if (manconv (d, from, to, &outbuf) == 0)
+		decompress_inprocess_replace (d, outbuf.buf, outbuf.len);
+	else {
+		/* manconv already wrote an error message to stderr.  Just
+		 * return non-zero.
+		 */
+		free (outbuf.buf);
+		ret = -1;
+	}
+
+	free (to);
+out:
+	gl_list_free (from);
+	return ret;
+}
