@@ -1,6 +1,6 @@
 /*
  * whatis.c: search the index or whatis database(s) for words.
- *  
+ *
  * Copyright (C) 1994, 1995 Graeme W. Wilford. (Wilf.)
  * Copyright (C) 2001, 2002, 2003, 2004, 2006, 2007, 2008, 2009, 2010, 2011,
  *               2012 Colin Watson.
@@ -21,11 +21,11 @@
  * along with man-db; if not, write to the Free Software Foundation,
  * Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- * routines for whatis and apropos programs. Whatis looks up the 
- * keyword for the description, apropos searches the entire database 
+ * routines for whatis and apropos programs. Whatis looks up the
+ * keyword for the description, apropos searches the entire database
  * for word matches.
  *
- * Mon Aug  8 20:35:30 BST 1994  Wilf. (G.Wilford@ee.surrey.ac.uk) 
+ * Mon Aug  8 20:35:30 BST 1994  Wilf. (G.Wilford@ee.surrey.ac.uk)
  *
  * CJW: Add more safety in the face of corrupted databases.
  */
@@ -34,6 +34,7 @@
 #  include "config.h"
 #endif /* HAVE_CONFIG_H */
 
+#include <assert.h>
 #include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
@@ -47,27 +48,29 @@
 #define _(String) gettext (String)
 #define N_(String) gettext_noop (String)
 
-#ifdef HAVE_ICONV
-#  include <iconv.h>
-#endif /* HAVE_ICONV */
-
 #include <sys/types.h>
 #include <sys/stat.h>
 #include "regex.h"
 
 #include "argp.h"
+#include "attribute.h"
 #include "dirname.h"
+#include "error.h"
 #include "gl_hash_set.h"
 #include "gl_list.h"
 #include "gl_xset.h"
 #include "fnmatch.h"
 #include "progname.h"
+#include "xalloc.h"
 #include "xvasprintf.h"
 
 #include "manconfig.h"
 
+#include "appendstr.h"
 #include "cleanup.h"
-#include "error.h"
+#include "debug.h"
+#include "fatal.h"
+#include "filenames.h"
 #include "glcontainers.h"
 #include "pipeline.h"
 #include "pathsearch.h"
@@ -76,10 +79,12 @@
 #include "xregcomp.h"
 #include "encodings.h"
 #include "sandbox.h"
+#include "util.h"
 
 #include "mydbm.h"
 #include "db_storage.h"
 
+#include "convert.h"
 #include "manp.h"
 
 static gl_list_t manpathlist;
@@ -92,11 +97,7 @@ bool am_apropos;
 int quiet = 1;
 man_sandbox *sandbox;
 
-#ifdef HAVE_ICONV
-iconv_t conv_to_locale;
-#endif /* HAVE_ICONV */
-
-static regex_t *preg;  
+static regex_t *preg;
 static bool regex_opt;
 static bool exact;
 
@@ -123,22 +124,29 @@ static const char args_doc[] = N_("KEYWORD...");
 static const char apropos_doc[] = "\v" N_("The --regex option is enabled by default.");
 
 static struct argp_option options[] = {
-	{ "debug",		'd',	0,		0,	N_("emit debugging messages") },
-	{ "verbose",		'v',	0,		0,	N_("print verbose warning messages") },
-	{ "regex",		'r',	0,		0,	N_("interpret each keyword as a regex"),	10 },
-	{ "exact",		'e',	0,		0,	N_("search each keyword for exact match") }, /* apropos only */
-	{ "wildcard",		'w',	0,		0,	N_("the keyword(s) contain wildcards") },
-	{ "and",		'a',	0,		0,	N_("require all keywords to match"),			20 }, /* apropos only */
-	{ "long",		'l',	0,		0,	N_("do not trim output to terminal width"),		30 },
-	{ "sections",		's',	N_("LIST"),	0,	N_("search only these sections (colon-separated)"),	40 },
-	{ "section",		0,	0,		OPTION_ALIAS },
-	{ "systems",		'm',	N_("SYSTEM"),	0,	N_("use manual pages from other systems") },
-	{ "manpath",		'M',	N_("PATH"),	0,	N_("set search path for manual pages to PATH") },
-	{ "locale",		'L',	N_("LOCALE"),	0,	N_("define the locale for this search") },
-	{ "config-file",	'C',	N_("FILE"),	0,	N_("use this user configuration file") },
-	{ "whatis",		'f',	0,		OPTION_HIDDEN,	0 },
-	{ "apropos",		'k',	0,		OPTION_HIDDEN,	0 },
-	{ 0, 'h', 0, OPTION_HIDDEN, 0 }, /* compatibility for --help */
+	OPT ("debug", 'd', 0, N_("emit debugging messages")),
+	OPT ("verbose", 'v', 0, N_("print verbose warning messages")),
+	OPT ("regex", 'r', 0, N_("interpret each keyword as a regex"), 10),
+	/* apropos only */
+	OPT ("exact", 'e', 0, N_("search each keyword for exact match")),
+	OPT ("wildcard", 'w', 0, N_("the keyword(s) contain wildcards")),
+	/* apropos only */
+	OPT ("and", 'a', 0, N_("require all keywords to match"), 20),
+	OPT ("long", 'l', 0, N_("do not trim output to terminal width"), 30),
+	OPT ("sections", 's', N_("LIST"),
+	     N_("search only these sections (colon-separated)"), 40),
+	OPT_ALIAS ("section", 0),
+	OPT ("systems", 'm', N_("SYSTEM"),
+	     N_("use manual pages from other systems")),
+	OPT ("manpath", 'M', N_("PATH"),
+	     N_("set search path for manual pages to PATH")),
+	OPT ("locale", 'L', N_("LOCALE"),
+	     N_("define the locale for this search")),
+	OPT ("config-file", 'C', N_("FILE"),
+	     N_("use this user configuration file")),
+	OPT_HIDDEN ("whatis", 'f'),
+	OPT_HIDDEN ("apropos", 'k'),
+	OPT_HELP_COMPAT,
 	{ 0 }
 };
 
@@ -241,7 +249,7 @@ static error_t parse_opt (int key, char *arg, struct argp_state *state)
 	return ARGP_ERR_UNKNOWN;
 }
 
-static char *help_filter (int key, const char *text, void *input _GL_UNUSED)
+static char *help_filter (int key, const char *text, void *input MAYBE_UNUSED)
 {
 	switch (key) {
 		case ARGP_KEY_HELP_PRE_DOC:
@@ -282,39 +290,6 @@ static char *locale_manpath (const char *manpath)
 	return new_manpath;
 }
 
-#ifdef HAVE_ICONV
-static char *simple_convert (iconv_t conv, char *string)
-{
-	if (conv != (iconv_t) -1) {
-		size_t string_conv_alloc = strlen (string) + 1;
-		char *string_conv = xmalloc (string_conv_alloc);
-		for (;;) {
-			char *inptr = string, *outptr = string_conv;
-			size_t inleft = strlen (string);
-			size_t outleft = string_conv_alloc - 1;
-			if (iconv (conv, (ICONV_CONST char **) &inptr, &inleft,
-				   &outptr, &outleft) == (size_t) -1 &&
-			    errno == E2BIG) {
-				string_conv_alloc <<= 1;
-				string_conv = xrealloc (string_conv,
-							string_conv_alloc);
-			} else {
-				/* Either we succeeded, or we've done our
-				 * best; go ahead and print what we've got.
-				 */
-				string_conv[string_conv_alloc - 1 - outleft] =
-					'\0';
-				break;
-			}
-		}
-		return string_conv;
-	} else
-		return xstrdup (string);
-}
-#else /* !HAVE_ICONV */
-#  define simple_convert(conv, string) xstrdup (string)
-#endif /* HAVE_ICONV */
-
 /* Do the old thing, if we cannot find the relevant database.
  * This invokes grep once per argument; we can't do much about this because
  * we need to know which arguments failed.  The only way to speed this up
@@ -325,6 +300,7 @@ static void use_grep (const char * const *pages, int num_pages, char *manpath,
 		      bool *found)
 {
 	char *whatis_file = xasprintf ("%s/whatis", manpath);
+	assert (whatis_file);
 
 	if (CAN_ACCESS (whatis_file, R_OK)) {
 		const char *flags;
@@ -352,8 +328,8 @@ static void use_grep (const char * const *pages, int num_pages, char *manpath,
 			else
 				anchored_page = xasprintf ("^%s", pages[i]);
 
-			grep_cmd = pipecmd_new_argstr (get_def_user ("grep",
-								     GREP));
+			grep_cmd = pipecmd_new_argstr
+				(get_def_user ("grep", PROG_GREP));
 			pipecmd_argstr (grep_cmd, flags);
 			pipecmd_args (grep_cmd, anchored_page, whatis_file,
 				      (void *) 0);
@@ -413,8 +389,8 @@ static struct mandata *resolve_pointers (MYDBM_FILE dbf, struct mandata *info,
 
 /* fill_in_whatis() is really a ../libdb/db_lookup.c routine but whatis.c
    is the only file that actually requires access to the whatis text... */
-   
-/* Take mandata struct (earlier returned from a dblookup()) and return 
+
+/* Take mandata struct (earlier returned from a dblookup()) and return
    the relative whatis */
 static char *get_whatis (struct mandata *info, const char *page)
 {
@@ -483,7 +459,7 @@ static void display (MYDBM_FILE dbf, struct mandata *info, const char *page)
 	} else
 		string = appendstr (string, whatis, "\n", (void *) 0);
 
-	string_conv = simple_convert (conv_to_locale, string);
+	string_conv = convert_to_locale (string);
 	fputs (string_conv, stdout);
 
 	free (string_conv);
@@ -505,10 +481,10 @@ static int do_whatis_section (MYDBM_FILE dbf,
 	int count = 0;
 
 	infos = dblookup_all (dbf, page, section, false);
-	GL_LIST_FOREACH_START (infos, info) {
+	GL_LIST_FOREACH (infos, info) {
 		display (dbf, info, page);
 		count++;
-	} GL_LIST_FOREACH_END (infos);
+	}
 	gl_list_free (infos);
 	return count;
 }
@@ -519,7 +495,7 @@ static bool suitable_manpath (const char *manpath, const char *page_dir)
 	gl_list_t page_manpathlist;
 	bool ret;
 
-	page_manp = get_manpath_from_path (page_dir, 0);
+	page_manp = get_manpath_from_path (page_dir, false);
 	if (!page_manp || !*page_manp) {
 		free (page_manp);
 		return false;
@@ -609,7 +585,7 @@ static bool all_set (int num_pages, bool *found_here)
 
 static void parse_name (const char * const *pages, int num_pages,
 			const char *dbname, bool *found, bool *found_here)
-{ 
+{
 	int i;
 
 	if (regex_opt) {
@@ -636,17 +612,17 @@ static void parse_name (const char * const *pages, int num_pages,
 }
 
 /* return true on word match */
-static bool _GL_ATTRIBUTE_PURE match (const char *page, const char *whatis)
+static bool ATTRIBUTE_PURE match (const char *page, const char *whatis)
 {
 	size_t len = strlen (page);
 	const char *begin;
 	char *p;
 
 	begin = whatis;
-	
+
 	/* check for string match, then see if it is a _word_ */
 	while (whatis && (p = strcasestr (whatis, page))) {
-		char *left = p - 1; 
+		char *left = p - 1;
 		char *right = p + len;
 
 		if ((p == begin || (!CTYPE (isalpha, *left) && *left != '_')) &&
@@ -660,7 +636,7 @@ static bool _GL_ATTRIBUTE_PURE match (const char *page, const char *whatis)
 
 static void parse_whatis (const char * const *pages, int num_pages,
 			  const char *whatis, bool *found, bool *found_here)
-{ 
+{
 	int i;
 
 	if (regex_opt) {
@@ -721,9 +697,7 @@ static void do_apropos (MYDBM_FILE dbf,
 	while (!end) {
 #endif /* !BTREE */
 		char *tab;
-		struct mandata info;
-
-		memset (&info, 0, sizeof (info));
+		struct mandata *info = NULL;
 
 		/* bug#4372, NULL pointer dereference in MYDBM_DPTR (cont),
 		 * fix by dassen@wi.leidenuniv.nl (J.H.M.Dassen), thanx Ray.
@@ -733,33 +707,38 @@ static void do_apropos (MYDBM_FILE dbf,
 		if (!MYDBM_DPTR (cont))
 		{
 			debug ("key was %s\n", MYDBM_DPTR (key));
-			error (FATAL, 0,
+			fatal (0,
 			       _("Database %s corrupted; rebuild with "
 				 "mandb --create"),
 			       dbf->name);
 		}
 
+#pragma GCC diagnostic push
+#if GNUC_PREREQ(10,0)
+#  pragma GCC diagnostic ignored "-Wanalyzer-use-after-free"
+#endif
 		if (*MYDBM_DPTR (key) == '$')
 			goto nextpage;
 
 		if (*MYDBM_DPTR (cont) == '\t')
 			goto nextpage;
+#pragma GCC diagnostic pop
 
 		/* a real page */
 
-		split_content (dbf, MYDBM_DPTR (cont), &info);
+		info = split_content (dbf, MYDBM_DPTR (cont));
 
 		/* If there are sections given, does any of them match
 		 * either the section or extension of this page?
 		 */
 		if (sections) {
 			char * const *section;
-			int matched = 0;
+			bool matched = false;
 
 			for (section = sections; *section; ++section) {
-				if (STREQ (*section, info.sec) ||
-				    STREQ (*section, info.ext)) {
-					matched = 1;
+				if (STREQ (*section, info->sec) ||
+				    STREQ (*section, info->ext)) {
+					matched = true;
 					break;
 				}
 			}
@@ -769,7 +748,7 @@ static void do_apropos (MYDBM_FILE dbf,
 		}
 
 		tab = strrchr (MYDBM_DPTR (key), '\t');
-		if (tab) 
+		if (tab)
 			 *tab = '\0';
 
 		memset (found_here, 0, num_pages * sizeof (*found_here));
@@ -778,30 +757,34 @@ static void do_apropos (MYDBM_FILE dbf,
 		if (am_apropos) {
 			char *whatis;
 
-			whatis = info.whatis ? xstrdup (info.whatis) : NULL;
+			whatis = info->whatis ? xstrdup (info->whatis) : NULL;
 			if (!combine (num_pages, found_here) && whatis)
 				parse_whatis (pages, num_pages,
 					      whatis, found, found_here);
 			free (whatis);
 		}
 		if (combine (num_pages, found_here))
-			display (dbf, &info, MYDBM_DPTR (key));
+			display (dbf, info, MYDBM_DPTR (key));
 
 		if (tab)
 			*tab = '\t';
 nextpage:
+#pragma GCC diagnostic push
+#if GNUC_PREREQ(10,0)
+#  pragma GCC diagnostic ignored "-Wanalyzer-double-free"
+#endif
 #ifndef BTREE
 		nextkey = MYDBM_NEXTKEY (dbf, key);
 		MYDBM_FREE_DPTR (cont);
 		MYDBM_FREE_DPTR (key);
-		key = nextkey; 
+		key = nextkey;
 #else /* BTREE */
 		MYDBM_FREE_DPTR (cont);
 		MYDBM_FREE_DPTR (key);
 		end = man_btree_nextkeydata (dbf, &key, &cont);
 #endif /* !BTREE */
-		info.addr = NULL; /* == MYDBM_DPTR (cont), freed above */
-		free_mandata_elements (&info);
+#pragma GCC diagnostic pop
+		free_mandata_struct (info);
 	}
 
 	free (found_here);
@@ -811,32 +794,23 @@ nextpage:
 static bool search (const char * const *pages, int num_pages)
 {
 	bool *found = XCALLOC (num_pages, bool);
-	char *catpath, *mp;
+	char *mp;
 	bool any_found;
 	int i;
 
-	GL_LIST_FOREACH_START (manpathlist, mp) {
-		char *database;
+	GL_LIST_FOREACH (manpathlist, mp) {
+		char *catpath, *database;
 		MYDBM_FILE dbf;
 
 		catpath = get_catpath (mp, SYSTEM_CAT | USER_CAT);
-		
-		if (catpath) {
-			database = mkdbname (catpath);
-			free (catpath);
-		} else
-			database = mkdbname (mp);
+		database = mkdbname (catpath ? catpath : mp);
 
 		debug ("path=%s\n", mp);
 
-		dbf = MYDBM_RDOPEN (database);
-		if (dbf && dbver_rd (dbf)) {
-			MYDBM_CLOSE (dbf);
-			dbf = NULL;
-		}
-		if (!dbf) {
+		dbf = MYDBM_NEW (database);
+		if (!MYDBM_RDOPEN (dbf) || dbver_rd (dbf)) {
 			use_grep (pages, num_pages, mp, found);
-			continue;
+			goto next;
 		}
 
 		if (am_apropos)
@@ -847,11 +821,12 @@ static bool search (const char * const *pages, int num_pages)
 			else
 				do_whatis (dbf, pages, num_pages, mp, found);
 		}
-		MYDBM_CLOSE (dbf);
-		free (database);
-	} GL_LIST_FOREACH_END (manpathlist);
 
-	chkr_garbage_detector ();
+next:
+		MYDBM_FREE (dbf);
+		free (database);
+		free (catpath);
+	}
 
 	any_found = false;
 	for (i = 0; i < num_pages; ++i) {
@@ -869,9 +844,6 @@ static bool search (const char * const *pages, int num_pages)
 int main (int argc, char *argv[])
 {
 	char *program_base_name;
-#ifdef HAVE_ICONV
-	char *locale_charset;
-#endif
 	int status = OK;
 
 	set_program_name (argv[0]);
@@ -915,7 +887,7 @@ int main (int argc, char *argv[])
 
 	read_config_file (user_config_file != NULL);
 
-	/* close this locale and reinitialise if a new locale was 
+	/* close this locale and reinitialise if a new locale was
 	   issued as an argument or in $MANOPT */
 	if (locale) {
 		free (internal_locale);
@@ -944,12 +916,6 @@ int main (int argc, char *argv[])
 
 	display_seen = new_string_set (GL_HASH_SET);
 
-#ifdef HAVE_ICONV
-	locale_charset = xasprintf ("%s//IGNORE", get_locale_charset ());
-	conv_to_locale = iconv_open (locale_charset, "UTF-8");
-	free (locale_charset);
-#endif /* HAVE_ICONV */
-
 	if (regex_opt) {
 		int i;
 		preg = XNMALLOC (num_keywords, regex_t);
@@ -968,10 +934,6 @@ int main (int argc, char *argv[])
 		free (preg);
 	}
 
-#ifdef HAVE_ICONV
-	if (conv_to_locale != (iconv_t) -1)
-		iconv_close (conv_to_locale);
-#endif /* HAVE_ICONV */
 	gl_set_free (display_seen);
 	free_pathlist (manpathlist);
 	free (manp);
